/* Programme chargé d'établir le dialogue avec le Moniteur USB de l'IDE.
   Les valeurs envoyées sont traitées par l'analyseur syntaxique.
   Retourne des accusés de réception. Les valeurs des consignes reçues
                       sont affichées sur OLED si elles sont correctes.
   >>>> Penser à initialiser le Moniteur à 115200 bauds. ( Ou revoir la valeur dans void setup().)
   
   Ce programme est conçu pour mettre au point le dialogue Homme/Machine ainsi que les
   protocoles d'alternat sur la ligne USB. Emule l'analyseur syntaxique, génère les codes d'erreur,
   accuse réception, et aiguille les actions à conduire sur consigne cohérente.
   La liste des caractères de commande valides est dans Traite_Consigne().
   
   Ce programme ne fait que traiter le dialogue avec le Moniteur de l'IDE et de renseigner
   l'état des paramètres qui en résultent sur l'afficheur OLED.

   UTILISER LE PROGRAMME : Frapper une chaine terminée par "Valide" sur la ligne série puis
       observer l'accusé de réception et comparer au tableau des erreurs de syntaxe;
       Toute chaîne sera limitée en saisie à sept caractères, les suivants seront ignorés.
       Tout caractère autre qu'un chiffre après la valeur numérique sera ignoré.
       (Par exemple proposer : "h45*ABCDEFGHIJKL" et Valider.
       Si on saisit une chaine exagérément longue il faut valider deux fois.
   
   Erreur 1 : Chaîne de longueur nulle. (Donc uniquement la sentinelle "[Valider]".)
   Erreur 2 : Caractère de commande invalide.
   Erreur 3 : Valeur numérique hors limites [-100 à +100].
   
   SDA est branché sur A4.  SCL est branché sur A5
                                                                PGM               RAM dynamique
 PGM de dialogue sur la ligne série.                    9782 Octets (31%)      564 Octets (27%)    */
 
//----------------------- Déclarations pour le matériel --------------------------
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//----------------------- Définition des broches d'E/S -------------------------
#define BUZZER 2 // Le bruiteur "Actif" est piloté par la sortie binaire D2.

//-------------------------- Constantes du programme ---------------------------
#define LGR_chaine 7 // Longueur maximale de la chaine de dialogue.
const byte TailleTexte = LGR_chaine + 1; // Taille réservée pour LGR_MAX plus le délimiteur.

//--------------------------- Variables du programme ---------------------------
boolean Ouverte;
char Caractere;
char Coude, Epaule, Hanche;
byte Compteur_caracteres;
byte num_ERR;
int Valeur_Numerique; // Contiendra la transformée numérique d'une chaine.
                      // Int pour pouvoir faire varier entre -90 et +90 et compatible string;

char Chaine_Memorisee[TailleTexte]; // Tampon mémoire pour la chaine entrante.

void setup() {
  u8g.setFont(u8g_font_6x10); // Pocice de matrices de 6x10 pixels lettres MJA/min comprises.
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  Serial.begin(115200); Serial.println("FIN avec [Valider]");
  Coude = 0; Epaule = 0; Hanche = 0;}

void loop() {
  PAGE1();
  Attendre_une_chaine();
  Serial.print('['); Serial.print(Caractere); Serial.print(Chaine_Memorisee); Serial.print("] > ");
  Analyseur_syntaxique();} // Traite la consigne.
  
void QUITTER() {
  // Ici il faudra placer le bras manipulateur en configuration "rétracté".
  Ouverte = true; // Pour ne pas contracter en permanence le ressorts.
  u8g.sleepOn();} // Place l'écran en veille.

void Traite_Consigne() {// Les commandes sont classées par ordre alphabétique.
  switch (Caractere) {
    case 'a' : {u8g.sleepOff(); break;} // Activer l'écran.
    case 'c' : {Coude = Valeur_Numerique; break;}
    case 'e' : {Epaule = Valeur_Numerique; break;}
    case 'f' : {Ouverte = false; break;} // Ferme la pince.
    case 'h' : {Hanche = Valeur_Numerique; break;}  
    case 'o' : {Ouverte = true; break;} // Ouvre la pince.
    case 'q' : {QUITTER(); break;} // laisser l'ensemble en configuration "propre".
    case 'v' : {u8g.sleepOn(); break;} // Passer l'écran en veille.
    default : num_ERR = 2;}}

void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void Analyseur_syntaxique() {// Compteur_caracteres mis à jour par Attendre_une_chaine()
  num_ERR = 0;
  Serial.print(" Nb = "); Serial.print(Compteur_caracteres);
  if (Compteur_caracteres == 0) num_ERR = 1;
  else {Compteur_caracteres--;
        //==================== Analyse de la commande =====================
  if ((Valeur_Numerique < -100) || (Valeur_Numerique > 100)) num_ERR = 3;
        if (num_ERR == 0) Traite_Consigne();} // Impérativement en premier car peut générer l'erreur 4.
  Serial.print(" C = "); Serial.print(Caractere); Serial.print(" Valeur = ");
  Serial.print(Valeur_Numerique); if (num_ERR == 0) Serial.println(" !OK!");
  else {BIP(); Serial.print(" !ERR "); Serial.print(num_ERR); Serial.println("!");}}

void Attendre_une_chaine() {
  boolean FINI; char Caractere_lu;
  FINI = false; Compteur_caracteres=0; // Si FIFO non vide lire les caractères :
  while(!FINI) {
    if (Serial.available()) {Caractere_lu = Serial.read();
        if (Caractere_lu == char(13)) FINI = true; // char (13) est le LF de "Valider". (Sentinelle.)
        if ((Compteur_caracteres < (LGR_chaine)) && (!FINI))
           {Chaine_Memorisee[Compteur_caracteres] = Caractere_lu; Compteur_caracteres++;
            delay(5);}}} // Délai impératif.
  Chaine_Memorisee[Compteur_caracteres] = '\0'; // Placer le délimiteur de fin de texte.
  Caractere = char(Chaine_Memorisee[0]);
  // Extraire le nombre. En sortie Valeur_numerique contiendra le nombre dans la chaine
  // ou 00 si le texte n'est pas "numérique" : 
  for(byte J = 0; J < Compteur_caracteres; J++)
      Chaine_Memorisee[J] = Chaine_Memorisee[J+1]; // Enlève le caractère de contrôle. 
  Valeur_Numerique = atoi(Chaine_Memorisee);}
  
void Affiche_valeur_angulaire(int Valeur) {
  if (Valeur == 0)  u8g.print(' '); if(abs(Valeur) < 10) u8g.print(' ');
  if (Valeur > 0)  u8g.print('+'); u8g.print(Valeur); u8g.print(" degre");}
 
void CADRES() {u8g.drawFrame(0,0,128,13); u8g.drawFrame(0,12,128,52);} // Cadre TITRE et cadre données.
  
void Aff_TEXTE(byte X, byte Y, char TEXTE[]) {// Choisir Y=9 pour tous les titres.
  char Caractere; byte I = 0; Caractere = 'x'; // 'x' pour ne pas avoir 0 en entrée de boucle.
  while (Caractere !=0) {Caractere = TEXTE[I]; u8g.setPrintPos((I*6)+X, Y); I++; u8g.print(Caractere);}}

void PAGE1() {
  u8g.setFont(u8g_font_6x10); // Pocice de matrices de 6x10 pixels lettres MJA/min comprises.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //------------------------------------------------------------------
  CADRES();
  Aff_TEXTE(11, 9 ,"Etat motorisation.");
  Aff_TEXTE(5,24,"  PINCE :"); if (Ouverte) Aff_TEXTE(72,24,"Ouverte"); else  Aff_TEXTE(72,23,"Fermee");
  Aff_TEXTE(5,35,"  COUDE :"); u8g.setPrintPos(60, 35); Affiche_valeur_angulaire(Coude);
  Aff_TEXTE(5,45," EPAULE :"); u8g.setPrintPos(60, 45); Affiche_valeur_angulaire(Epaule);
  Aff_TEXTE(5,55," HANCHE :"); u8g.setPrintPos(60, 55); Affiche_valeur_angulaire(Hanche);}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.
