/* Programme identique à P02_Pilotage_Moniteur_USB mais ajoute les commandes suivantes :
   mN : Mémorise l'état actuel du bras manipulateur dans l'enregistrement N avec N = [1 à 50].
   rN : Restore la configuration enregistrée dans l'enregistrement N avec N = [1 à 50].
   Z : Efface sans avertissement tous les enregistrements. (Impose MAJ suivi de 50 pour éviter une erreurs.)
   l : Liste en Hexadécimal le contenu actuel de la mémoire EEPROM.
   u : Liste les enregistrements actuellement Utilisés.
   kN : Efface l'enregistrement N. (Kill.)
   sN : Angle de serrage de la pince. (Compris entre 30° et 90°.)
   L'écran affiche en haut à droite le nombre d'enregistrements en EEPROM.
   
   Ce programme ne fait que traiter le dialogue avec le Moniteur de l'IDE et de renseigner
   l'état des paramètres qui en résulte sur l'afficheur OLED. (Servomoteurs non traités.)

   UTILISER LE PROGRAMME : Frapper une chaine sur la ligne série puis observer l'accusé de réception
   et comparer au tableau des erreurs de syntaxe; Les contraintes pour la chaîne saisie sont identiques
   à celles du programme P02.
   
   Erreur 1 : Chaîne inférieure à un caractères. (Donc chaîne vide.)
   Erreur 2 : Caractère de commande invalide.
   Erreur 3 : Consigne de position angulaire hors de la fourchette.
   Erreur 4 : Num enregistrement hors de la fourchette [1 à 50].
   Erreur 5 : Tenter de restituer un enregistrement vide.
   Erreur 6 : Commande Z non suivie de la valeur 50.
   
   SDA est branché sur A4.  SCL est branché sur A5
                                                                PGM               RAM dynamique
 PGM de dialogue sur la ligne série.                    11362 Octets (36%)      673 Octets (32%)

----------------------- Déclarations pour le matériel --------------------------    */
#include <EEPROM.h>
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

//----------------------- Définition des broches d'E/S -------------------------
#define BUZZER 2 // Le bruiteur est piloté par la sortie binaire D2.

//-------------------------- Constantes du programme ---------------------------
#define Version_PGM "11-10-2019"    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
#define LGR_chaine 7 // Longueur maximale de la chaine de dialogue.
const byte TailleTexte = LGR_chaine + 1; // Taille réservée pour LGR_MAX plus le délimiteur.

//--------------------------- Variables du programme ---------------------------
boolean Ouverte;

char Angle_Serrage_Pince;
char Caractere;
char Coude;
char Epaule;
char Hanche;
int DataX; // Entier prévu en EEPROM pour éventuelle modification.

byte Compteur_caracteres;
byte Nb_Enregistrements;
byte num_ERR;
byte PGM_actuel;
int Valeur_Numerique; // Contiendra la transformée numérique d'une chaine.
int PTR;

char Chaine_Memorisee[TailleTexte]; // Tampon mémoire pour la chaine entrante.
//------------------------------------------------------------------------------

void setup() {
  u8g.setFont(u8g_font_6x10); // Pocice de matrices de 6x10 pixels lettres MJA/min comprises.
  pinMode(BUZZER, OUTPUT); digitalWrite(BUZZER, LOW);
  Serial.begin(115200); Serial.print("version du "); Serial.println(Version_PGM);
  Angle_Serrage_Pince = 60; Coude = 0; Epaule = 0; Hanche = 0;
  Calcule_Nb_Enregistrements(); PAGE1();}

void loop() {
  Attendre_une_chaine();
  Serial.print('['); Serial.print(Caractere); Serial.print(Chaine_Memorisee); Serial.print("] > ");
  Analyseur_syntaxique(); // Traite la consigne.
  PAGE1();}
  
void QUITTER() {
  // Ici il faudra placer le bras manipulateur en configuration "rétracté".
  Ouverte = true; // Pour ne pas contracter en permanence le ressorts.
  u8g.sleepOn();} // Place l'écran en veille.

void Traite_angle_de_serage_pince() {
  if ((Valeur_Numerique < 30) || (Valeur_Numerique > 90)) num_ERR = 3;
  if (num_ERR == 0) {Angle_Serrage_Pince = Valeur_Numerique; PAGE1();}}
  
void Traite_Consigne() {// Les commandes sont classées par ordre alphabétique.
  switch (Caractere) {
    case 'a' : {u8g.sleepOff(); break;} // Activer l'écran.
    case 'c' : {Coude = Valeur_Numerique; break;}
    case 'e' : {Epaule = Valeur_Numerique; break;}
    case 'f' : {Ouverte = false; break;} // Ferme la pince.
    case 'h' : {Hanche = Valeur_Numerique; break;}
    case 'k' : {Kill_enregistrement(); break;} //Effacer un enregistrement.
    case 'l' : {Lister_EEPROM(); break;}
    case 'm' : {Memorise_Posture(); break;}
    case 'o' : {Ouverte = true; break;} // Ouvre la pince.
    case 'x' : {DataX = Valeur_Numerique; break;}
    case 'q' : {QUITTER(); break;} // Laisser l'ensemble en configuration "propre".
    case 'r' : {Traite_Restitution(); break;}
    case 's' : {Traite_angle_de_serage_pince(); break;}
    case 'u' : {Liste_les_enregistrements_utilises(); break;}
    case 'v' : {u8g.sleepOn(); break;} // Passer l'écran en veille.
    case 'Z' : {Efface_toutes_les_postures_en_EEPROM(); break;}
    default : num_ERR = 2;}}

void BIP() {digitalWrite(BUZZER, HIGH); delay(100); digitalWrite(BUZZER, LOW);}

void Analyseur_syntaxique() {// Compteur_caracteres mis à jour par Attendre_une_chaine()
  num_ERR = 0;
  Serial.print(" Nb = "); Serial.print(Compteur_caracteres);
  if (Compteur_caracteres == 0) num_ERR = 1;
  else {Compteur_caracteres--;
        //==================== Analyse de la commande =====================
  if ((Valeur_Numerique < -100) || (Valeur_Numerique > 100)) num_ERR = 3;
        if (num_ERR == 0) Traite_Consigne();} // Impérativement en premier car peut générer l'erreur 4.
  Serial.print(" C = "); Serial.print(Caractere); Serial.print(" Valeur = ");
  Serial.print(Valeur_Numerique); if (num_ERR == 0) Serial.println(" !OK!");
  else {BIP(); Serial.print(" !ERR "); Serial.print(num_ERR); Serial.println("!");}}

void Attendre_une_chaine() {
  boolean FINI; char Caractere_lu;
  FINI = false; Compteur_caracteres=0; // Si FIFO non vide lire les caractères :
  while(!FINI) {
    if (Serial.available()) {Caractere_lu = Serial.read();
        if (Caractere_lu == char(13)) FINI = true; // char (13) est le LF de "Valider". (Sentinelle.)
        if ((Compteur_caracteres < (LGR_chaine)) && (!FINI))
           {Chaine_Memorisee[Compteur_caracteres] = Caractere_lu; Compteur_caracteres++;
            delay(5);}}} // Délai impératif.
  Chaine_Memorisee[Compteur_caracteres] = '\0'; // Placer le délimiteur de fin de texte.
  Caractere = char(Chaine_Memorisee[0]);
  // Extraire le nombre. En sortie Valeur_numerique contiendra le nombre dans la chaine
  // ou 00 si le texte n'est pas "numérique" : 
  for(byte J = 0; J < Compteur_caracteres; J++)
      Chaine_Memorisee[J] = Chaine_Memorisee[J+1]; // Enlève le caractère de contrôle. 
  Valeur_Numerique = atoi(Chaine_Memorisee);}
  
void Affiche_valeur_angulaire(int Valeur) {
  if (Valeur == 0)  u8g.print(' '); if(abs(Valeur) < 10) u8g.print(' ');
  if (Valeur > 0)  u8g.print('+'); 
  u8g.print(Valeur); u8g.print(" degre");}

void CADRES() {u8g.drawFrame(0,0,128,13); u8g.drawFrame(0,12,128,52);} // Cadre TITRE et cadre données.
  
void Aff_TEXTE(byte X, byte Y, char TEXTE[]) {// Choisir Y=9 pour tous les titres.
  char Caractere; byte I = 0; Caractere = 'x'; // 'x' pour ne pas avoir 0 en entrée de boucle.
  while (Caractere !=0) {Caractere = TEXTE[I]; u8g.setPrintPos((I*6)+X, Y); I++; u8g.print(Caractere);}}
  
void PAGE1() {
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  CADRES(); u8g.drawVLine(85,0, 12 ); u8g.setColorIndex(0); u8g.drawHLine(95,63, 8); u8g.setColorIndex(1); 
  Aff_TEXTE(8, 10 ,"Etat du BRAS  Nb");  u8g.setPrintPos(109, 10); u8g.print(Nb_Enregistrements);
  Aff_TEXTE(5,21,"  PINCE :");
  if (Ouverte) Aff_TEXTE(72,21,"Ouverte");
  else {Aff_TEXTE(65,21,"Fermee "); u8g.setPrintPos(104, 21); u8g.print('a');
        u8g.setPrintPos(113, 21); u8g.print(int(Angle_Serrage_Pince));}
  Aff_TEXTE(5,31," Data X :"); u8g.setPrintPos(60, 31); Affiche_valeur_angulaire(DataX);
  Aff_TEXTE(5,41,"  COUDE :"); u8g.setPrintPos(60, 41); Affiche_valeur_angulaire(Coude);
  Aff_TEXTE(5,51," EPAULE :"); u8g.setPrintPos(60, 51); Affiche_valeur_angulaire(Epaule);
  Aff_TEXTE(5,61," HANCHE :"); u8g.setPrintPos(60, 61); Affiche_valeur_angulaire(Hanche);}
  //====================================================================
  while(u8g.nextPage());} // Fin de la boucle d'affichage de la page écran.

void Calcule_Nb_Enregistrements() {
  Nb_Enregistrements = 0;
  PTR = 474;
  while (PTR < 1023) {if (Lire_un_Octet_en_EEPROM()) Nb_Enregistrements++; PTR = PTR + 10;}}
  
void Efface_toutes_les_postures_en_EEPROM() {
  // Efface également les 5 programmes éventuels, car anticipe une version future.
  if (Valeur_Numerique != 50) num_ERR = 6;
  if (num_ERR == 0) {
      for (int I=459; I < 1024; I++) Ecrire_Octet_EEPROM(I, 00);
      Nb_Enregistrements = 0; PGM_actuel = 0;}}
  
//--------------------------- Routines de gestion EEPROMe ---------------------------
byte Lire_un_Octet_en_EEPROM() {PTR++; return eeprom_read_byte((unsigned char*) PTR - 1);}
     
int Lire_un_Entier_en_EEPROM() {PTR++; PTR++; return eeprom_read_word((unsigned int*) (PTR - 2));}

void Ecrire_un_Octet_en_EEPROM(byte Octet) {eeprom_write_byte((unsigned char*) PTR, Octet); PTR++;}

void Ecrire_un_Entier_en_EEPROM(int Entier)
     {eeprom_write_word((unsigned int*) PTR, Entier); PTR++; PTR++;}

void Verifie_1_a_50() {if ((Valeur_Numerique < 1) || (Valeur_Numerique > 50)) num_ERR = 4;}

void Positionne_PTR_en_fonction_de_Num() {PTR = 463; PTR = PTR + (11 * Valeur_Numerique);}
  
void Memorise_Posture() {
  Verifie_1_a_50();  // Num va de 1 à 50.
  if (num_ERR == 0) {
      // Les enregistrements commencent en adresse relative 474.
      Positionne_PTR_en_fonction_de_Num(); // Pointe le début de l'enregistrement.
      Ecrire_un_Octet_en_EEPROM(true); // valide l'enregistrement.
      Ecrire_un_Octet_en_EEPROM(Ouverte);
      Ecrire_un_Entier_en_EEPROM(DataX); Ecrire_un_Entier_en_EEPROM(Coude);
      Ecrire_un_Entier_en_EEPROM(Epaule); Ecrire_un_Entier_en_EEPROM(Hanche);
      Calcule_Nb_Enregistrements();}} // Calcule et affiche l'écran 1.
  
void Traite_Restitution() {
  Verifie_1_a_50();
  if (num_ERR == 0) {
     Positionne_PTR_en_fonction_de_Num(); // Pointe le début de l'enregistrement.
     if (!Lire_un_Octet_en_EEPROM()) num_ERR = 5; // Vérifie que l'enregistrement n'est pas vide.
     if (num_ERR == 0) {
         Ouverte = Lire_un_Octet_en_EEPROM(); DataX = Lire_un_Entier_en_EEPROM();
         Coude = Lire_un_Entier_en_EEPROM(); Epaule = Lire_un_Entier_en_EEPROM();
         Hanche = Lire_un_Entier_en_EEPROM(); PAGE1();}}} 

void Liste_les_enregistrements_utilises() {
  boolean Trouve = false;
  PTR = 474; Compteur_caracteres = 1; Serial.println();
  while (PTR < 1023) {
    if (Lire_un_Octet_en_EEPROM()) {Serial.print(Compteur_caracteres); Serial.println(" valide."); Trouve = true;}
    PTR = PTR + 10; Compteur_caracteres++;}
    if (!Trouve) {BIP(); Serial.println("Aucun enregistrement.");}}
    
void Kill_enregistrement() {// Ne fait que forcer à "00 le premier octet du groupe.
  Verifie_1_a_50(); Positionne_PTR_en_fonction_de_Num(); // Pointe le début de l'enregistrement.
  if (num_ERR == 0) {Ecrire_un_Octet_en_EEPROM(0); Calcule_Nb_Enregistrements();}}
  
//-----------------------------------------------------------------------------------

void Lister_EEPROM() {
  PTR = 0;
  Lister_un_BLOC();Lister_un_BLOC();
  Lister_un_BLOC();Lister_un_BLOC(); }
  
void Lister_un_BLOC() {
  Lister_ENTETE();
  byte Compteur_lignes = 0;
  while (Compteur_lignes < 16)
    {Lister_une_LIGNE(); Compteur_lignes++; } }
  
void Lister_une_LIGNE() {   
  byte Compteur_Octets = 0; Serial.print(' ');
  if (PTR<10) {Serial.print('0');};
  if (PTR<100) {Serial.print('0');};
  if (PTR<1000) {Serial.print('0');};
  Serial.print(PTR); Serial.print("  ");  
  while (Compteur_Octets < 16) {
    Lister_OCTET(); Compteur_Octets++; }
  Serial.println(); }
  
void Lister_ENTETE() {
   Serial.println();
   Serial.println(" ADRS  00 01 02 03 04 05 06 07 08 09 0A 0B 0C 0D 0E 0F");}
   
byte Lire_Octet_EEPROM(int ADRESSE) {return eeprom_read_byte((unsigned char*) ADRESSE);}

void Ecrire_Octet_EEPROM(int ADRESSE, byte Octet) {eeprom_write_byte((unsigned char*) ADRESSE, Octet);}
   
void Lister_OCTET() {
  byte octet = Lire_Octet_EEPROM(PTR);
  if (octet < 16) Serial.print('0'); 
  Serial.print(octet,HEX);
  Serial.print(' '); PTR++; }






