/* Petit programme pour tester le multiplexeur 16 canaux ADAFRUIT.

   Permet de déterminer les caractéristiques des moteurs. Pour couvrir à coup sûr la
   plage des 180° les impulsions étant volontairement un peu hors limites.
   Par défaut sur le multiplexeur l'adresse I2C sera 0x40.
   (Peut être modifiée sur le matériel mais dans cette application ce ne sera pas utile.)

   UTILISER LE PROGRAMME : En faisant varier la tension sur A3 avec un potentiomètre linéaire dont
       la valeur peut être comprise entre 4,7k à 22k ohm on pilote tous les canaux simultanément.
       Ainsi on peut brancher un sevomoteur ou une LED (Avec sa résistance de limitation de courant.) sur
       n'importe quelle sortie pour vérifier son bon fonctionnement et surtout déterminer les valeurs
       limites à imposer au moteur pour le maintenir dans la plage mécanique et électrique normale.
       La valeur de consigne est affichée en permanence sur l'afficheur OLED.
       (On peut ainsi vérifier la compatibilité entre les deux périphériques I2C.)
         
DIFFERENTS BRANCHEMENTS :
 D13 : Pilotage du disjoncteur s'il le test est effectué sur la machine entièrement terminée.
 A3 : Tention potentiométrique de commande. (0 à +5Vcc.)
 A4 -> SDA pour la ligne ligne I2C.
 A5 -> SCL pour la ligne ligne I2C.

                                                                PGM               RAM dynamique
 PGM avec affichage sur l'écran OLED.                     10682 Octets (34%)      463 Octets (22%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h> // Gère les protocole I2C sur A4 et A5.
#include "U8glib.h"
U8GLIB_SH1106_128X64 u8g(U8G_I2C_OPT_NONE);	// Usage de la ligne I2C (Two Wire Interface)

#include <Adafruit_PWMServoDriver.h>
Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//--------------------- Définition des broches d'E/S ------------------------
#define Entree_mesuree 17 // Entrée analogique A3 utilisée.
#define DISJONCTEUR 13 // Disjoncteur. (Coupe ou rétablit la puissance électrique.)
//------------------------ Constantes du programme --------------------------
#define NB_MAX  16 // Nombre de "sorties servomoteurs" branchées.

//------------------------- Variables du programme --------------------------
boolean Moteurs_ON;
byte Num_moteur;

int CNA;
//-------------------------- Fin des déclarations ---------------------------

void setup() {
  u8g.setFont(u8g_font_6x10); // Police de matrices de 6x10 pixels lettres MJA/min comprises. 
  pwm.begin(); // Initialise le module multiplexeur  
  pwm.setPWMFreq(50); // Fréquence de la PWM : En standard 50Hz pour des servomoteurs.
  pinMode(DISJONCTEUR, OUTPUT); digitalWrite(DISJONCTEUR, HIGH);
  Moteurs_ON = true;} // Anticipe la mise en veille éventuelle des moteurs.

void loop() {
 // Commander les 16 sorties simultanément à la même valeur.
  CNA = analogRead(Entree_mesuree);
  CNA = map(CNA, 0, 1023, 120, 520); // Génère une valeur comprise entre 130 et 500.
  u8g.firstPage(); do {// Début de la boucle d'affichage de la page écran.
  //================ Définition des éléments à afficher ================
  u8g.setPrintPos(20, 30); u8g.print("Consigne = "); u8g.print(CNA);}
  //====================================================================
  while(u8g.nextPage()); // Fin de la boucle d'affichage de la page écran.
  Num_moteur = 0;
  while (Num_moteur < NB_MAX) {Mouvoir(Num_moteur, CNA); Num_moteur ++;}}
  
void Mouvoir(byte Sortie, int Consigne) {if (Moteurs_ON) pwm.setPWM(Sortie,0, Consigne);}
      

  

  
